"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamicRequireSupportPlugin = void 0;
const tslib_1 = require("tslib");
const utils_1 = require("@morjs/utils");
const path_1 = tslib_1.__importDefault(require("path"));
/**
 * 实现异步 require 支持
 * 参见: https://developers.weixin.qq.com/miniprogram/dev/reference/api/require.html
 * 由于异步 require 可能请求的是分包中的文件, 会导致 webpack 无法执行静态解析的逻辑
 * 这里通过让 webpack 对 require('filepath', callback) 的调用不做处理来兼容
 * NOTE: 当前的做法有待进一步改进，目前仅仅是忽略的动态引用的处理, 需要进一步处理编译支持
 */
class DynamicRequireSupportPlugin {
    constructor() {
        this.name = 'DynamicRequireSupportPlugin';
    }
    apply(runner) {
        const warnDynamicRequire = function (parser, requireType) {
            var _a, _b, _c, _d;
            const fileName = ((_c = (_b = (_a = parser === null || parser === void 0 ? void 0 : parser.state) === null || _a === void 0 ? void 0 : _a.module) === null || _b === void 0 ? void 0 : _b.nameForCondition) === null || _c === void 0 ? void 0 : _c.call(_b)) || '';
            if (fileName) {
                utils_1.logger.warnOnce(`文件 ${path_1.default.relative((_d = runner.userConfig) === null || _d === void 0 ? void 0 : _d.srcPath, fileName)} 中包含异步 ${requireType}，将跳过静态解析，请自行保证引用路径的正确`);
            }
        };
        // require('filepath', callback) 支持
        const parseDynamicRequire = (parser) => {
            parser.hooks.call.for('require').tap(this.name, (expression) => {
                if (expression.type === 'CallExpression') {
                    // 如果参数数量为 1 则跳过
                    // 暂不检查第一个参数可能为三目运算的情况
                    if (expression.arguments.length <= 1)
                        return;
                    // 从第二个参数开始检查，发现为非 undefined 的值
                    // 则认为当前 require 为异步 require
                    for (let i = 1; i < expression.arguments.length; i++) {
                        const argument = expression.arguments[i];
                        // 如果是 void x 同样认为是 undefined
                        if (argument.type === 'UnaryExpression' &&
                            argument.operator === 'void') {
                            continue;
                        }
                        if (argument.type === 'Identifier' &&
                            argument.name === 'undefined') {
                            continue;
                        }
                        warnDynamicRequire(parser, 'require');
                        return true;
                    }
                }
            });
            // require.async('filepath') 支持
            parser.hooks.callMemberChainOfCallMemberChain
                .for('require')
                .tap(this.name, (expression, calleeMembers) => {
                var _a;
                if (expression.type === 'CallExpression' &&
                    ((_a = calleeMembers === null || calleeMembers === void 0 ? void 0 : calleeMembers.includes) === null || _a === void 0 ? void 0 : _a.call(calleeMembers, 'async'))) {
                    warnDynamicRequire(parser, 'require.async');
                    return true;
                }
            });
        };
        runner.hooks.compiler.tap(this.name, (compiler) => {
            compiler.hooks.normalModuleFactory.tap(this.name, (factory) => {
                factory.hooks.parser
                    .for('javascript/auto')
                    .tap(this.name, parseDynamicRequire);
                factory.hooks.parser
                    .for('javascript/dynamic')
                    .tap(this.name, parseDynamicRequire);
                factory.hooks.parser
                    .for('javascript/esm')
                    .tap(this.name, parseDynamicRequire);
            });
        });
    }
}
exports.DynamicRequireSupportPlugin = DynamicRequireSupportPlugin;
//# sourceMappingURL=dynamicRequireSupportPlugin.js.map